#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
//
//                Programmer: James A. Green
//                Update: Aug 22, 2003 - For avg luminosity in star age calc.
//
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .
//
//  starz4ag.cpp - Stellar Age, Evolution, Perturbations,
//                 Core Collapse, Scenarios, Precision depth scanning.
//
//
//  PROJECT STARMAN -
//	   starman.prj is the project file
//	       producing starman.exe from
//		     starz7.cpp - main module.
//		      starz7ag.cpp - Stellar ages and evolution, perturbations.
//		      starz7au.cpp - Neutrino luminosity & stellar wind.
//		      starz7fu.cpp - basic functions.
//		      starz7bo.cpp - bolometric corrections.
//		      starz7op.cpp - supplemental ops.
//
//               Greenwood Research, http://grnwd.tripod.com.
//
//  ROUTINES in starz4ag.cpp:
//
//    evolver() - Reports to file fp, Star Age,
//                Evolutionary Perturbations,
//                Core Collapse, White Dwarf formation,
//                Planetary Nebula ejection, valid for
//                lower main sequence < 4 solar masses,
//                as soon as our CNO H-burning code is perfect.
//                Presently valid for stars < 2 solar masses,
//                as the carbon cycle is tiny then, and p-p
//                fusion dominates.
//
//    dwarfradius(dwarfmass) - Calculates the radius of a
//                             White Dwarf as a funcion of its mass from 
//                            S.Chandrasehar table using linear interpolation.
//    dwarfm(double dwarfr) - Calculates the mass of a White Dwarf
//                            as a function of its radius.
//
//    depthscanner(void) - Depth Scanner requires a seperate program. Stub.
//
//  STAR AGE ESTIMATION & PERTURBATION ANALYSIS
//  - Model perturbed by mu(r) variance
//    resulting from the consumption of the core.
//    Temperature varies as mu(r).
//  
//  Compute the age of the star from the radius of the hydrogen-burning core,
//  the average core density, the observed lumininosity of the star, and
//  delh, the change in the hydrogen abundance required to normalize Chiu p-p fusion.
//  delh = 0.5976 - chiunormx is the change in the hydrogen abundance reqd
//  to normalize the Chiu p-p fusion for Clayton-Aller 1961 abundances.   <A> - Age Section.
//
//  starage0 = delh*(ro_core/(4.0*m_H))*(4*pi/3.0)((rhcore)**3)*(R_sun**3)*delE/L(R) sec.
//
//        L(R) is the stellar luminosity in watts. 1 year = 3.156e7 sec.
//        Assumes constant stellar luminosity = present luminosity.
//
//  starage1 - The 2nd star age uses the 1st-order average luminosity since star formation.
//
//   EXTERNAL VARIABLES
//
extern FILE *fp;                         // file pointer fp to starman.txt
extern int status;                       // file status
extern int dwarfformation;        // Enables white dwarf formation & envelope ejection report.
extern double cuberoot(double v);        // cuberoot(v)
extern double nthpower(double v, int n); // v**n

extern int looplimset2(int i);         // loop limit for starform[0]
extern int starform[1];            // 0->n=3, 1->n=1.5 polytrope specifier.
extern double theta[28];            // n=3   Cosmogenic polytropes, gamma=4/3,
extern double xr[28];               // n=3   P=P_gas plus P_radiation.
extern double xxr[23];              // n=1.5  Classical monatomic gas polytrope
extern double xtheta[23];           // n=1.5  P is gas pressure only, no radiation pressure.
extern double chiuluminosity[1];    // screened Chiu luminosity
extern double cnoluminosity[1];    //  Schwarzschild CNO luminosity
extern double sccnoluminosityrv;    // the screened (CNO) cycle luminosity, Hubert Reeves
extern double pi;		   //  pi, 3.14159.
extern double gnewton;		   // Newtons constant of gravitation
extern double msun;                // mass of the Sun in kg.
extern double xmstar[1];           // mass of the star in kg.
extern double zage;                // Zombeck stellar lifetime.
extern double ageschw;             // Schwarzschild star age.
extern double age;                 // Greens Main Sequence Lifetime.
extern double hkage;               // Helmholtz contraction time.
extern double chiunormx;           // X(H) to normalize L_nuclear_Chiu to L(R) observed.
extern double claynormx;           // X(H) to normalize L_nuclear_Clayton to L(R) observed.
extern double chemx[1];            // X(H) - initial hydrogen abundance by weight.
extern double chemy[1];            // Y(He) - initial helium abundance by weight.
extern double chemz[1];            // Z(hvy) - initial heavy element abundance by weight.
extern double chemmu[1];                 // initial molecular weight.
extern double xtc[1];                    // initial central temperature.
extern double rocoremin;                 // minimum density at far edge of H-burning core.
extern double xdensitycentral[1];        // max density at center of H-burning core.
extern double mh;                        // mass of the hydrogen atom in kg.
extern double rhcore;			 // radius of the core in solar radii.
extern double mshell[30];                // mass of the mth core shell.
extern double glomcore;                  // mass of the H-burning core in Kg.
extern double rsun;                      // radius of the Sun in meters.
extern double totalluminosity;          // total nuclear luminosity P-P, CNO, He-burning
extern double totalhluminosity;         // total P-P plus CNO H-burning, using Schw.CNO
extern double totalhluminosity2;        // total P-P plus CNO H-burning, using Reeves.CNO
extern int nucmixflag;                  // 1->use Schw.CNO, 2->Use Reeves CNO
//                                         3->Use scaled Scw.CNO, 4->Use Scaled Reeves CNO.
extern double bclratio[1];              // L_star/L_sun
extern double lsun;                     // Luminosity of the Sun in Watts
extern double xrstar[1];                // Radius of the Sun in meters.
extern double photontime;               // The stellar photon random-walk diffusion time.
extern double lightyears[1];            // Original Distance in lightyears for depthscanner.
extern void probe(int i);               // Probe identifies self as ith, waits for keystroke.

//
//
//    evolver() - Reports to file fp, Star Age,
//                Evolutionary Perturbations,
//                Core Collapse, White Dwarf formation,
//                Planetary Nebula ejection, valid for
//                lower main sequence < 4 solar masses,
//                as soon as our CNO H-burning code is perfect.
//                Presently valid for stars < 2 solar masses,
//                as the carbon cycle is tiny then, and p-p
//                fusion dominates.
//
//   Aug 22, 2003 - The age of the Sun may be improved by computing the
//                  the average luminosity since the Sun formed, and using
//                  this instead of the present luminosity. This will shorten
//                  age somewhat, since it has been declining in luminosity from
//                  from the nuclear luminosity we at first compute. For the Sun, consider
//                  lumavg = (totalluminosity + 1.0)/2.0  solar luminosities.
//                  For any star,  if totalluminosity > Lstar, use
//                  lumavg = (totalluminosity + lstar)/2.0, else use lumavg = lstar.
//                  bclratio[0] is the Lstar_observed/Lsun ratio that is multiplied below by
//                  lsun to get the luminosity of the star in Watts for the age calc.
//                  So we use (bclratio[0] + totalnuclum)/2.0 in place of
//		    bclratio[0] if totalnuclum > bclratio[0], else use bclratio[0].
//
//
void evolver(void)
{                           // local variables
	
	int kk;             // core scanning counter variable
	double dwradius(double dwarfmass);  // provides 2nd r-measure for collapsed core.
	double potenwhite2; // potential energy according to 2nd r-measure.
	double lenratio2;   // launch energy ratio according to 2nd r-measure
	double tt;          // temperature in core dimensions scan
	double rr;          // radius in core dimensions scan.
	double dwarfradius(double dwarfmass); // Compute collapsed core radius from core mass.
	double delh;             //  the change in X(H) required to normalize
//                                   so that L_nuclear equals L(R) observed.
	double dele;             // potenwhite/2.0 - launchenergy is kinetic energy.
	double kboltz = 1.3805e-23;  // Boltzmann's constant in J/K.
	double c = 2.9979e8;     // the speed of light in meters per second.
	double freefalltime;    // The core free fall collapse time, sec.
	double menvel;		// mass of the envelope around the H-burning core.
	int looplim;            // loop limit for starform[0], polytrope specifier.
	double corephotonmfp;   // core photon mean free path
	double coresurfacetemp; // core surface temperature
	double coreti;         //  core mean internal temperature
	double corephotontime;  // core photon diffusion time
	double vmax;            // ideal, lossless velocity of ejection envelope at infinity.
	double rocoreavg;        //  the average density of the H-burning core.
	double masscorekg;         //  the mass of the H-burning core, kg.
	double masscoresolar;     //  mass of the H-burning core in solar units.
	double massenvelratio; //  mass of the star ejected by w.dwarf formation in Star units.
	double masscoreratio;    //   mass H-burning core/star_mass.
	double corepot; // Better numerically integrated core potential to replace below.
	double corepoten;        // Potential energy of the core, Integral (GM(r)/r)dM,
//                        using  the linear approx ro(r) = ro(0) - ((ro(0) - ro(1))/r_core)*r,
//        is 4*pi*G*(ro(0)**2)/2  + 4*pi*G*((ro(1) - ro(0))/r_core)*((r_core)**3)/3.
	double potenwhite;       // approx. white dwarf remnant potential energy
//          (6.0/7.0)*G*M*M/(0.01*R_sun), n=1.5 polytrope
	double deltenvel;  // Max temperature rise in stellar envelope after core collapse
	double nparticles;  // Number of particles in the stellar envelope.
	double totalnuclum;    // Total nuclear luminosity for age calc, temporary var.
	double launchenergy;     // Energy to put envelope around core back to infinity.
	double lenratio;         // Launch energy ratio:
//                                  Available core collapse energy/Envelope_to_infinity energy.
// 	                            Only half of the white dwarf potential energy
//                                  is available for launching the envelope.
	double ageratio;         // Stellar lifetime measure over H-burning core age.
	double roc0, roc1;       //  H-burning core central density roc0 to final ro roc1.
	double ro;               // density in core-scanning for romin & CoreRadius rhcore.
	double roin,roout,romid; // inner, outer & middle shell densities in core mass integral.
	double rcore;            //  radius of the core in meters, rhcore*rsun;
        double phi;
	double starage;          //  the age of the star in years.
	double secperyear = 3.156e7;   // seconds per year.
	double corecollapsetime; //  Helmholtz-Kelvin collapse time of the core to W.D.state.
	double chemmucore;       //  molecular weight of the core.
	double mufinal;          //  final molecular weight in H-burning core.
	double drr;              //  delta R over R_star, for entire star from core delta_r.
	double delrr;            //  differential dr in core mass integration.
	double dpp;              //  delta_p over p in the H-burning core
	double dtt;              //  delta_t over t for the H-burning core temperature.
	double coreratio;         // r_core/R_star for the H-burning core.
	double muratio;           // delta_mu/mu_original for the H-burning core.
	double collapsetime;      // time from now until H-burning core collapse in years.
	double hburnlifetime;     // Total Main Sequence lifetime from core collapse model, yrs.
	double deltarratio;       // MAX(deltarratio/R_star_original) for star growth
//                                   from fusion function. (mu_avg/mu_original)**4
//                                   raises the temperature. Then the star expands
//                      until 1 = ((R_star_original/R)**6)*(mu_avg/mu_original)**4
//                      brings the p-p fusion function back to its old value if the
//		        star is oscillating, and to some R < R_max such that
//                      1 < ((R_star_original/R)**6)*(mu_avg/mu_original)**4
//                      to merely grow. See the supplemental notes to
//			Thermonuclear Fusion in Stars for the theoretical underpinning.

//  H-burning core measurements and Stellar Age calculation

            kk=0;
	    rhcore  = 0.0;
	    looplim = looplimset2(0);	    
	    while(kk < looplim){
               if(starform[0] == 0){
	          tt = xtc[0]*theta[kk];
		  rr = (xrstar[0]/rsun)*(xr[kk]/6.9011);
	       }
	       else if(starform[0] == 1){
		  tt = xtc[0]*xtheta[kk];
		  rr = (xrstar[0]/rsun)*(xxr[kk]/3.65375);
	       }
	       if(tt > 1.0e7){		  
		  rhcore = rr;   // Finds hydrogen-burning core maximum radius r/R_sun.
	       }      
       
               kk++;
	    }

//    Find H-burning core density at edge, compute mshell[k], coremass and core potential

	    kk = 0;
	    while(kk < 30){   // initialize shell masses to non-zero value for limit probs,
	       mshell[kk] = 10000.0;  // then recompute them below. Shell masses are used
	       kk++;                  // for fusion rate per unit mass from totlum[ ]
	    }                         // and must be non-zero for proper zero at boundary.

	    kk = 1;
	    looplim = looplimset2(0);
	    if(starform[0] == 0){   //  n = 3
	      delrr = (xrstar[0]/rsun)*(xr[1]/6.9011);
	      ro = xdensitycentral[0];
	      masscorekg =  (4.0/3.0)*3.24159*(delrr*rsun)*(delrr*rsun)*(delrr*rsun)*ro;
	      mshell[0] = masscorekg;
	      corepot = gnewton*masscorekg/delrr;
	    }
	    if(starform[0] == 1){  //  n = 1.5.
	      delrr = (xrstar[0]/rsun)*(xr[1]/3.63375);
	      ro = xdensitycentral[0];
	      masscorekg =  (4.0/3.0)*3.24159*(delrr*rsun)*(delrr*rsun)*(delrr*rsun)*ro;
	      mshell[0] = masscorekg;
	      corepot = gnewton*masscorekg/delrr;
	    }
	    while(kk < looplim - 1){
	      if(starform[0] == 0){ // gamma = 4/3 model
		  roin = xdensitycentral[0]*theta[kk]*theta[kk]*theta[kk];
		  roout = xdensitycentral[0]*theta[kk+1]*theta[kk+1]*theta[kk+1];
		  romid = (roin + roout)/2.0;
		  tt = xtc[0]*theta[kk];
		  delrr = (xrstar[0]/rsun)*(xr[kk+1]/6.9011) - (xrstar[0]/rsun)*(xr[kk]/6.9011);
		  rr = (xrstar[0]/rsun)*(xr[kk]/6.9011);
		  rr = (rr + (xrstar[0]/rsun)*(xr[kk+1]/6.9011))/2.0;
		  if(tt > 1.0e7){
		     rocoremin = ro; // find rocoremin for hyrogen-burning core.
		     corepot = corepot + gnewton*4.0*3.24159*(rr*rsun)*(rr*rsun)*(delrr*rsun)*romid/rr;
		     mshell[kk] = 4.0*3.24159*(rr*rsun)*(rr*rsun)*(delrr*rsun)*romid;
		     masscorekg = masscorekg + mshell[kk];
		  } // Compute mshell[kk] everywhere.
		  mshell[kk] = 4.0*3.24159*(rr*rsun)*(rr*rsun)*(delrr*rsun)*romid;
	      }
	      else if(starform[0] == 1){
		  phi = sqrt(xtheta[kk]); // use ID-tagged sqrt()
		  tt = xtc[0]*xtheta[kk];
		  roin = xdensitycentral[0]*phi*phi*phi;
		  phi = sqrt(xtheta[kk+1]);
		  roout = xdensitycentral[0]*phi*phi*phi;  //  n = 1.5.
		  romid = (roin + roout)/2.0;
		  delrr = (xrstar[0]/rsun)*(xr[kk+1]/3.63375) - (xrstar[0]/rsun)*(xr[kk]/3.63375);
		  rr = (xrstar[0]/rsun)*(xr[kk]/3.63375);
		  rr = (rr + (xrstar[0]/rsun)*(xr[kk+1]/3.63375))/2.0;
		  if(tt > 1.0e7){
		     rocoremin = ro; // find rocoremin for hyrogen-burning core.
		     corepot = corepot + gnewton*4.0*3.24159*(rr*rsun)*(rr*rsun)*(delrr*rsun)*romid/rr;
		     mshell[kk] = 4.0*3.24159*(rr*rsun)*(rr*rsun)*(delrr*rsun)*romid;
		     masscorekg = masscorekg + mshell[kk];
		  } // Compute mshell[kk] everywhere.
		  mshell[kk] = 4.0*3.24159*(rr*rsun)*(rr*rsun)*(delrr*rsun)*romid;
	      }
	      kk++;
	    }
	    glomcore = masscorekg; // load global core mass glomcore.

//    Find X in H-burning core such that Lnuc/Lstar = 1 when p-p fusion & CNO are both
//    accounted for.  Assume helium fusion is negligible in the Main Sequence stars.
//    Note that then Lnuc/Lstar = totalluminosity/lratio.
//
//  DEBUG probe
//
//	    printf("DEBUG: rhcore = %f, solar radii.\n", rhcore);
//	    printf("       rhcore*rsun/xrstar[0] = %f, .\n", (rhcore*rsun)/xrstar[0]);
//	    printf("       xdensitycentral[0] = %e kg per m**3\n", xdensitycentral[0]);
//	    printf("       rocoremin = %e kg per m**3\n", rocoremin);
//	    printf("       rocoreavg = %e kg per m**3\n", rocoreavg);
//	    probe(0);
//
//  END DEBUG probe
//
//	    masscoresolar = masscorekg/msun;   // mass of the H-burning core, solar units.
//	    masscoreratio = masscorekg/xmstar[0];   // M_core/M_star .
//	    fprintf(fp,"    H-burningCoreMass = %e kg , = %f Msun , Mcore/Mstar = %f.\n",
//	    masscorekg, masscoresolar, masscoreratio);	    
//	    delh = chemx[0];  //   Compute H-burning core lifetime
//	    corelifetime  = (1.0/(4.0*mh))*delh*masscorekg;
//	    corelifetime = corelifetime*(4.19737e-12)/(lsun*bclratio[0]);  // 4.19737e-12 J is 26.2 MeV.
//	    corelifetime = corelifetime/3.156e7;  // Find years age from seconds.

//
//
//    Chiu Star Age and pertubation analysis.
//
	    fprintf(fp,"\n  STAR AGE ESTIMATION & MODEL PERTURBATION ANALYSIS\n\n");

	    rcore = rhcore*rsun; // radius of the H-burning core in meters.
//                            Must include CNO luminosity for stars with M > 2 M_sun
//
	    if(nucmixflag == 1 || nucmixflag == 3){
	       chiunormx = chemx[0]/sqrt(((chiuluminosity[0] + cnoluminosity[0])/bclratio[0]));

	       if( ((chiuluminosity[0] + cnoluminosity[0])/bclratio[0]) < 1.0 )
	          delh = 0.0; // Watch out for Lnuc < Lstar, when
	       else                            // the star is unborn.
	          delh = chemx[0] - chiunormx;
	    }
	    else if(nucmixflag == 2 || nucmixflag == 4){
	       chiunormx = chemx[0]/sqrt(((chiuluminosity[0] + sccnoluminosityrv)/bclratio[0]));

	       if( ((chiuluminosity[0] + sccnoluminosityrv)/bclratio[0]) < 1.0 )
	          delh = 0.0; // Watch out for Lnuc < Lstar, when
	       else                            // the star is unborn.
	          delh = chemx[0] - chiunormx;
	    }
	    starage  = (1.0/(4.0*mh))*delh*masscorekg;
	    starage = starage*(4.19737e-12)/(lsun*bclratio[0]);  // 4.19737e-12 J is 26.2 MeV.
	    starage = starage/3.156e7;  // Find years age from seconds.
	    fprintf(fp,"Chiu Star age_BEST = %e yr based on the H-burning core\n",starage);
	    fprintf(fp,"  depletion required to normalize L_nuclear(R) for Chiu p-p fusion,\n");
	    fprintf(fp,"  assumes constant luminosity at present luminosity.\n");

	    if(bclratio[0] < (chiuluminosity[0] + sccnoluminosityrv)){
	      starage  = (1.0/(4.0*mh))*delh*masscorekg;
	      starage = starage*(4.19737e-12)/(lsun*( ((bclratio[0] + chiuluminosity[0] + sccnoluminosityrv)/2.0)) );  // 4.19737e-12 J is 26.2 MeV.
	      starage = starage/3.156e7;  // Find years age from seconds.	    
	      fprintf(fp,"Chiu Star age_ERR = %e yr based on the H-burning core\n",starage);
	      fprintf(fp,"  depletion required to normalize L_nuclear(R) for Chiu p-p fusion,\n");
	      fprintf(fp,"  uses 1st-order average luminosity since star formation,\n");
	      fprintf(fp,"  making erroneous constant-radius assumption.\n");
	    }

	    if(totalluminosity < bclratio[0]){  // Linker flags automatic variable limit 6-14-2002
	       fprintf(fp,"  Total nuclear luminosity < L(R), Chiu Star Age invalid. Options:\n");
//	       fprintf(fp,"    1)Star is contracting to Main Seq.\n");
//	       fprintf(fp,"    2)Star is a red giant.\n");
//	       fprintf(fp,"    3)Nominal distance too large.\n");
//	       fprintf(fp,"    4)Alternative energy source, or CNO off somewhat.\n");
	    }

	    fprintf(fp,"  10**7 K H-burning core radius = %1.2f solar radii",rhcore);
	    fprintf(fp," = %1.2f star radius.\n", rhcore*rsun/xrstar[0]); // Core size

	    fprintf(fp,"  Mantle: X=%f, Y=%f, Z=%f, mu=%f.\n",
	      chemx[0],chemy[0],chemz[0],chemmu[0]);          //  Evolution of core chemistry
	    chemmucore = 1.0/(2.0*(chemx[0] - delh) + (0.75*(chemy[0] + delh)) + 0.5*chemz[0]);
	    fprintf(fp,"  Core:   X=%f, Y=%f, Z=%f, mu=%f.\n",
	      chemx[0] - delh ,chemy[0] + delh, chemz[0], chemmucore);
	    
	    masscoresolar = masscorekg/msun;   // mass of the H-burning core, solar units.
	    masscoreratio = masscorekg/xmstar[0];   // M_core/M_star .
	    massenvelratio = 1.0 - masscoreratio;  // Find M_ejectae/M_star.
	    fprintf(fp,"  Core_Mass = %e kg , = %f M_sun , M_core/M_star = %f.\n",
	      masscorekg, masscoresolar, masscoreratio);

	    roc0 = xdensitycentral[0] ;  // Calculate the stored potential energy of the
	    roc1 = rocoremin;            // H-burning core in Joules. Estimate! Replace
//                                          with integral over core.
//	    corepoten = 4.0*pi*gnewton*roc0*(rcore*rcore)/2.0; // Uses linear approximation.
//	    corepoten = corepoten + (4.0*pi*gnewton*(roc1 - roc0)*(rcore*rcore)/3.0);
//                                  Old core linearized core potential cal above replaced.

	    corepoten = corepot; // Use more exact integral form of core potential energy.
	    fprintf(fp,"  Core gravitational potential energy = %e Joules.\n", corepoten);

//              Perturbation of Stellar Dimensions due to transformation of molecular weight.
//				1) Naive gas ball analysis.

	    coreratio = rhcore*rsun/xrstar[0];   // cal rcore/R_star
	    muratio = 1.0 + ( ((chemmucore - chemmu[0])/chemmu[0])*nthpower(coreratio,3) );
	    fprintf(fp,"  1)Perturbation analysis based on nuclear energy gen & volume-avg mu. Disfavored.\n");
	    fprintf(fp,"    Old T_c = %e K, Perturbed (mu_avg/mu_old)*T_c = %e K\n",
	       xtc[0], xtc[0]*muratio);
	    deltarratio = cuberoot( muratio*muratio ) - 1.0;

	    fprintf(fp,"    Radial expansion perturbation delta_R/R_star < %f.\n", deltarratio);

//              Perturbation of Stellar Dimensions due to transformation of molecular weight.
//				2) Polytropic analysis, gas plus radiation pressure case.

	    fprintf(fp,"  2)Polytropic analysis based on gas plus radiation laws. Since formation:\n");
	    fprintf(fp,"    Pressure and Temp drop in core as mu rises, R_star decreases.\n");
	    drr = -(1.0/3.0)*((chemmucore - chemmu[0])/chemmucore)*rhcore;
	    fprintf(fp,"      delta_R/R = -(1/3)(del_mu/mu)(R_core/R) = %f.\n", drr);
	    dtt = -((chemmucore - chemmu[0])/chemmucore);
	    fprintf(fp,"      delta_T/T = -(del_mu/mu) = %f.\n", dtt);
	    dpp = -(4.0/3.0)*((chemmucore - chemmu[0])/chemmucore);
	    fprintf(fp,"      delta_P/P = -(4/3)(del_mu/mu) = %f.\n", dpp);
	    mufinal = 1.0/( (0.75*(chemx[0] + chemy[0])) + 0.5*chemz[0] );

	    fprintf(fp,"    When the core has exhausted its hydrogen, mu = %f, and\n", mufinal);
	    drr = -(1.0/3.0)*((mufinal - chemmu[0])/mufinal)*rhcore;
	    fprintf(fp,"      delta_R/R = -(1/3)(del_mu/mu)(R_core/R)= %f.\n", drr);
	    dtt = -((mufinal - chemmu[0])/mufinal);
	    fprintf(fp,"      delta_T/T = -(del_mu/mu) = %f.\n", dtt);
	    dpp = -(4.0/3.0)*((mufinal - chemmu[0])/mufinal);
	    fprintf(fp,"      delta_P/P = -(4/3)(del_mu/mu) = %f.\n", dpp);

//
//  Clayton age & perturbation analysis of star, including Reeves screening CNO.
//
	    if(nucmixflag == 1 || nucmixflag == 3) // Check for Schw.CNO, burns H.
	       totalnuclum = totalhluminosity;
	    if(nucmixflag  == 2 || nucmixflag == 4) // Check for Reeves CNO, burns H.
	       totalnuclum = totalhluminosity2; // Main seq.lifetime depends on H-burning core.

	    claynormx = chemx[0]/sqrt((totalnuclum/bclratio[0]));  
	    if(totalnuclum/bclratio[0] <= 1.0)  // Watch out for Lnuc < Lstar,
	       delh = 0.0;                         // when core abundances remain constant
	    else                                  //  and the star is unborn.
	       delh = chemx[0] - claynormx;
	    starage  = (1.0/(4.0*mh))*delh*masscorekg;
	    starage = starage*(4.19737e-12)/(lsun*bclratio[0]);  // 4.19737e-12 J is 26.2 MeV.
	    starage = starage/3.156e7;  // Find years age from seconds.
	    
	    fprintf(fp,"\nClayton Star age_BEST = %e yr based on the H-burning core\n",starage);
	    fprintf(fp,"  depletion required to normalize L_nuclear(R) for Clayton p-p fusion,\n");
	    fprintf(fp,"  assumes present luminosity since star formation.\n");

	    if(bclratio[0] < totalnuclum){
              starage  = (1.0/(4.0*mh))*delh*masscorekg;
	      starage = starage*(4.19737e-12)/(lsun*( ((bclratio[0] + totalnuclum)/2.0) ));  // 4.19737e-12 J is 26.2 MeV.
	      starage = starage/3.156e7;  // Find years age from seconds.
	      fprintf(fp,"Clayton Star age_ERR = %e yr based on the H-burning core\n",starage);
	      fprintf(fp,"  depletion required to normalize L_nuclear(R) for Chiu p-p fusion,\n");
	      fprintf(fp,"  uses 1st order average luminosity since star formation,\n");
	      fprintf(fp,"  making erroneous constant radius assumption.\n");
	    }
	    if(totalluminosity < bclratio[0]){
	       fprintf(fp,"  Total nuclear luminosity < L(R), Clayton Star Age invalid. Options:\n"); 
	    }

	    fprintf(fp,"  10**7 K H-burning core radius = %1.2f solar radii", rhcore);
	    fprintf(fp," = %1.2f star radius.\n", rhcore*rsun/xrstar[0]);
	    fprintf(fp,"  Mantle: X=%f, Y=%f, Z=%f, mu=%f.\n",
	      chemx[0],chemy[0],chemz[0],chemmu[0]);
	    chemmucore = 1.0/(2.0*(chemx[0] - delh) + (0.75*(chemy[0] + delh)) + 0.5*chemz[0]);
	    fprintf(fp,"  Core:   X=%f, Y=%f, Z=%f, mu=%f.\n",
	      chemx[0] - delh, chemy[0] + delh, chemz[0], chemmucore);
	    
	    
	    masscoresolar = masscorekg/msun;   // mass of the H-burning core, solar units.
	    masscoreratio = masscorekg/xmstar[0];   // M_core/M_star .
	    fprintf(fp,"  H-burningCoreMass = %e kg , = %f Msun , Mcore/Mstar = %f.\n",
	      masscorekg, masscoresolar, masscoreratio);

//                        Redo core potential energy above from core integral.
	    fprintf(fp,"  Core gravitational potential energy = %e Joules.\n", corepoten);

	    coreratio = rhcore*rsun/xrstar[0];   // cal rcore/R_star
	    muratio = 1.0 + ( ((chemmucore - chemmu[0])/chemmu[0])*nthpower(coreratio,3) );
	    fprintf(fp,"  1)Perturbation analysis based on nuclear energy gen & volume-avg mu. Disfavored.\n");
	    fprintf(fp,"    Old T_c = %e K, Perturbed (mu_avg/mu_old)*T_c = %e K\n",
	       xtc[0], xtc[0]*muratio);
	    deltarratio = cuberoot( muratio*muratio ) - 1.0;
	    fprintf(fp,"    Radial expansion perturbation delta_R/R_star < %f.\n", deltarratio);

	    fprintf(fp,"  2)Polytropic analysis based on gas plus radiation laws. Since formation:\n");
	    drr = -(1.0/3.0)*((chemmucore - chemmu[0])/chemmucore)*rhcore;
	    fprintf(fp,"      delta_R/R = -(1/3)(del_mu/mu)(R_core/R)= %f.\n", drr);
	    dtt = -((chemmucore - chemmu[0])/chemmucore);
	    fprintf(fp,"      delta_T/T = -(del_mu/mu) = %f.\n", dtt);
	    dpp = -(4.0/3.0)*((chemmucore - chemmu[0])/chemmucore);
	    fprintf(fp,"      delta_P/P = -(4/3)(del_mu/mu) = %f.\n", dpp);

	    mufinal = 1.0/( (0.75*(chemx[0] + chemy[0])) + 0.5*chemz[0] );
	    fprintf(fp,"    When the core has exhausted its hydrogen, mu = %f, and\n", mufinal);
	    drr = -(1.0/3.0)*((mufinal - chemmu[0])/mufinal)*rhcore;
	    fprintf(fp,"      delta_R/R = -(1/3)(del_mu/mu)(R_core/R)= %f.\n", drr);
	    dtt = -((mufinal - chemmu[0])/mufinal);
	    fprintf(fp,"      delta_T/T = -(del_mu/mu) = %f.\n", dtt);
	    dpp = -(4.0/3.0)*((mufinal - chemmu[0])/mufinal);
	    fprintf(fp,"      delta_P/P = -(4/3)(del_mu/mu) = %f.\n", dpp);
//
// Collapse of the H-burning core, Planetary Nebula ejection, and White Dwarf formation.
//
//   SUN:According to subsequent calculations, the ejection of the planetary nebula
//       will change the orbit of the Earth such that T = 4673.31 days = 12.794826 years.
//       The orbit around the white dwarf will be at 3.57698 AU from the solar relic.
//       The derivation proceeds via conservation of angular momentum.
//
	 if(dwarfformation == 1){
	    masscoresolar = masscorekg/msun;
	    fprintf(fp,"\nTHE HYDROGEN-BURNING CORE - COLLAPSE & ENVELOPE EJECTION PROBLEM\n\n", dpp);
	    fprintf(fp,"The smallest white dwarf in the Chandrasekhar Table has a mass\n");
	    fprintf(fp,"of 0.22 solar masses. Sun core mass is 0.279565 solar masses.\n",masscoresolar);
	    fprintf(fp,"Bowers & Deeming: All stars < 4 solar masses form a white dwarf.\n");
	    fprintf(fp,"Smallest observed white dwarfs: M/Msun = 0.36 for 40 Eri B.\n");
	    fprintf(fp,"                                M/Msun = 0.39 for W 1346.\n");
	    fprintf(fp,"                                M/Msun = 0.01 for Van Maanen 2.\n");
	    fprintf(fp,"Two radial measures are used below for the collapsed core:\n");
	    fprintf(fp,"  1) Theoretical, after table by Chandrasekhar with 1.44 Msun limit.\n");
	    fprintf(fp,"  2) Experimental curve-fit, dwarf mass-radius, from Bowers & Deeming.\n");
	    claynormx = chemx[0]/sqrt((totalnuclum/bclratio[0])); // totalnuclum depends on nucmixflag.
	    if(claynormx > chemx[0] )  // Watch out for Lnuc < Lstar,
	       delh = chemx[0];        // when core abundances remain constant
	    else                       // and the star is unborn.
	       delh =  claynormx;      // Countdown to core collapse.
	      
	    collapsetime  = (1.0/(4.0*mh))*delh*masscorekg;
	    collapsetime = collapsetime*(4.19737e-12)/(lsun*bclratio[0]);  // 4.19737e-12 J is 26.2 MeV.
	    collapsetime = collapsetime/3.156e7;  // Find years age from seconds
	    masscoresolar = masscorekg/msun;
	    fprintf(fp,"Hydrogen-burning core mass = %f solar masses.\n", masscoresolar);
	    if(masscoresolar > 0.64)
	      fprintf(fp,"Core > 0.64 solar masses -> Turns on Helium fusion to halt collapse. ELSE:\n");
	    fprintf(fp,"  M_core/M_star = %f , if > 0.12, H-depleted core can collapse.\n", masscorekg/xmstar[0]);
	    if(masscorekg/xmstar[0] > 0.12)
		 fprintf(fp,"  M_core/M_star > M_Schoenberg-Chandrasekhar/M_star for collapse =.12.\n");
	    else
		 fprintf(fp,"  M_core/M_star <= M_Schoenberg-Chandrasekhar/M_star for collapse =.12.\n");
	    fprintf(fp,"  Core gravitational potential energy = %e Joules.\n", corepoten);

	    potenwhite = (6.0/7.0)*gnewton*masscorekg*masscorekg/(dwarfradius(masscorekg/msun)*rsun);
	    potenwhite2 = (6.0/7.0)*gnewton*masscorekg*masscorekg/(dwradius(masscorekg/msun)*rsun);

	    fprintf(fp,"  White dwarf approx. remnant potential energy = %e Joules.\n",potenwhite);
	    fprintf(fp,"   using 2nd radius measure: remnant potential energy = %e Joules.\n",potenwhite2);
	    launchenergy = (6.0/7.0)*gnewton*xmstar[0]*xmstar[0]/(xrstar[0]);
	    launchenergy = launchenergy - corepoten;
	    fprintf(fp,"  Stellar envelope launch energy for infinity = %e Joules.\n", launchenergy);
	    lenratio = ((potenwhite - corepoten)/2.0)/launchenergy;
	    lenratio2 = ((potenwhite2 - corepoten)/2.0)/launchenergy;
	    fprintf(fp,"  Questionable:assumes half of the collapse energy is locked into dwarf thermal E_th, ala\n");
	    fprintf(fp,"  the virial theorem, then available E_total/2 divided between envelope E_th & kinetic E_kin.\n", lenratio);
	    fprintf(fp,"  Total_available_collapse_energy/envelope_launch energy = %f \n", lenratio);
	    fprintf(fp,"   2nd r-measure Total_avail_collapse_energy/envelope_launch energy = %f \n", lenratio2);
	    fprintf(fp,"  Assume virial theorem applies in a reciprocal form, so that half of the\n");
	    fprintf(fp,"  available collapse energy E_total/4 can expand the shell kinetically. Then:\n");
	    if(lenratio > 2.0){
	       fprintf(fp,"    The stellar shell expands to infinity when core forms white dwarf.\n");
	       menvel = xmstar[0] - masscorekg;
	       dele = ((potenwhite - corepoten)/2.0)/2.0 - launchenergy;	       
	       if(2.0*dele/menvel > 0.0){   // for the launch of the exterior shell.
		 vmax = sqrt(2.0*dele/menvel); // cal max velocity of ejectae.
	       }
	       else{
		 vmax = - 0.1;
		 fprintf(fp,"  Maximum velocity of ejectae at infinity invalid!\n");
	       }
	       fprintf(fp,"  Maximum velocity of ejectae at infinity = %e m/sec,\n", vmax);
	       vmax = (vmax/1.4e11)*3.156e7;
	       fprintf(fp,"                      = %f Astronomical_Units/year,\n", vmax);
	       fprintf(fp,"                      = %f LY/year, ", vmax*1.581395349e-5);
	       fprintf(fp,"or  = %f years/LY,\n", 1.0/(vmax*1.581395349e-5));
	       vmax = 364.25/vmax;  // Compute days to reach Earth.
	       fprintf(fp,"                      = %f days to reach 1 AU,\n", vmax);
	       fprintf(fp,"  assuming half energy is wasted in heating the envelope.\n");
	       fprintf(fp,"  %f percent of the stellar mass is ejected.\n", massenvelratio*100.0);
	    }
	    else{
	       fprintf(fp,"  Too little energy is present to expand the envelope to infinity.\n");
	    }
	    if(lenratio2 > 2.0){
	       fprintf(fp,"    2nd r - The stellar shell expands to infinity when core forms white dwarf.\n");
	       menvel = xmstar[0] - masscorekg;
	       dele = ((potenwhite2 - corepoten)/2.0)/2.0 - launchenergy;
	       
	       if(2.0*dele/menvel > 0.0){   // for the launch of the exterior shell.
		 vmax = sqrt(2.0*dele/menvel); // cal max velocity of ejectae.
	       }
	       else{
		 vmax = - 0.1;
		 fprintf(fp,"  2nd Maximum velocity of ejectae at infinity invalid!\n");
	       }
	       fprintf(fp,"  2nd Maximum velocity of ejectae at infinity = %e m/sec,\n", vmax);
	       vmax = (vmax/1.4e11)*3.156e7;
	       fprintf(fp,"                      = %f Astronomical_Units/year 2nd,\n", vmax);
	       fprintf(fp,"                      = %f LY/year, 2nd, ", vmax*1.581395349e-5);
	       fprintf(fp,"= %f yr/LY,\n", 1.0/(vmax*1.581395349e-5));
	       vmax = 364.25/vmax;  // Compute days to reach Earth.
	       fprintf(fp,"                      = %f days to reach 1 AU, 2nd\n", vmax);
	       fprintf(fp,"  assuming half energy is wasted in heating the envelope, 2nd.\n");
	       fprintf(fp,"  %f percent of the stellar mass is ejected.\n", massenvelratio*100.0);
	    }
	    else{
	       fprintf(fp,"  Too little energy is present to expand the envelope to infinity.\n");
	    }
	    nparticles = (xmstar[0] - masscorekg)/(chemmu[0]*mh); // Cal number of particles in envelope.

	    if((2.0*(potenwhite - corepoten)/4.0)/(nparticles*kboltz*3.0) > 0.0 ){
	       deltenvel = 2.0*((potenwhite - corepoten)/4.0)/(nparticles*kboltz*3.0);
	    }
	    else{
	       fprintf(fp,"  Envelope heating calc invalid!\n");
	    }
	    fprintf(fp,"  Half of the core collapse energy goes to heat the core,(virial th.)\n");
	    fprintf(fp,"  the other half is available to heat the envelope and eject it.\n");
	    fprintf(fp,"  Assume 1/2 of this available core collapse energy goes to heat the envelope.\n");
	    fprintf(fp,"  Then the max temperature rise in the envelope =%e K.\n", deltenvel);

	    if( 2.0*((potenwhite2 - corepoten)/4.0)/(nparticles*kboltz*3.0) > 0.0 ){
	       deltenvel = 2.0*((potenwhite2 - corepoten)/4.0)/(nparticles*kboltz*3.0);
	    }
	    else{
	       fprintf(fp,"  Envelope heating calc invalid! 2nd r.\n");
	    }
	    fprintf(fp,"  Half of the core collapse energy goes to heat the core,(virial th.)\n");
	    fprintf(fp,"  the other half is available to heat the envelope and eject it.\n");
	    fprintf(fp,"  Assume 1/2 of this available core collapse energy goes to heat the envelope.\n");
	    fprintf(fp,"  Then the max temperature rise in the envelope =%e K, 2nd.\n", deltenvel);

	    corecollapsetime =  ((potenwhite - corepoten)/2.0)/(bclratio[0]*lsun);
	    corecollapsetime = corecollapsetime/secperyear;
	    fprintf(fp,"  The core collapse time controls the vigor of the shell expansion.\n");
	    fprintf(fp,"  Helmholtz-Kelvin model core collapse time =%e yr,\n", corecollapsetime);
	    fprintf(fp,"    but observationally planetary nebula ejection lasts < 3,000 yr,\n");
	    fprintf(fp,"    and the nebula clears away in say, 25,000 yrs.\n");    

	    freefalltime = rhcore*rhcore*rhcore*rsun*rsun*rsun/((6.67e-11)*masscorekg);
	    freefalltime = 1.11072*sqrt(freefalltime); // The Physics of Stars, A.C.Phillips.
	    freefalltime = freefalltime/60.0;  // A.C.Phillips free-fall time, pg.7.
	    fprintf(fp,"  Core freefall time = %e min \n", freefalltime);
	    fprintf(fp,"     = %e hrs = %e days = %e years.\n",
	      freefalltime/60.0, freefalltime/1440.0, freefalltime/526176.0 );
	    fprintf(fp,"  Stellar photon random-walk escape time = %e sec \n", photontime); // Phillips, p.23.
	    fprintf(fp,"    = %e years.\n", photontime/3.156e7);

// calculate mean free path from core opacity next time.
//
//	    rocoreavg = (xdensitycentral[0] + rocoremin)/2.0;
//	    coresurfacetemp = xtc[0]*cuberoot(rocoremin/xdensitycentral[0]);
//	    coreti = xtc[0]*cuberoot(rocoreavg/xdensitycentral[0]);
//	    corephotonmfp =   coresurfacetemp/coreti;  // Compute photon mean free path.
//	    corephotonmfp = nthpower(corephotonmfp,4);
//	    corephotonmfp = corephotonmfp*rhcore*rsun;
//	    corephotontime = (rhcore*rsun*rhcore*rsun)/(c*corephotonmfp);
//	    fprintf(fp,"  Core photon mean free path = %e meters ", corephotonmfp); // Phillips, p.23.
//	    fprintf(fp,"= %e mm\n", corephotonmfp*1000.0);
//	    fprintf(fp,"  Core photon random-walk escape time = %e sec \n", corephotontime); // Phillips, p.23.
//	    fprintf(fp,"    = %e years.\n", corephotontime/3.156e7);
//
//   Canonical Stellar Lifetimes and the Lifetime of the H-burning core.
//
	    if(masscoresolar > 0.64){
	      fprintf(fp,"Core > 0.64 Msun -> Turns on Helium fusion to halt core collapse,\n");
	      fprintf(fp,"possibly initiating radial stellar pulsations.\n");
	    }
	    fprintf(fp,"Time from now to core collapse in H-burning core: = %e yr.\n", collapsetime);
	    hburnlifetime = collapsetime + starage;
	    fprintf(fp,"Total Main-Sequence Lifetime until core collapse: = %e yr,\n", hburnlifetime);
	    fprintf(fp,"   according to our simplistic H-burning core collapse theory.\n");
	    ageratio = hburnlifetime/zage;
	    fprintf(fp,"                   H-burning_lifetime/Zombeck_Age = %f \n", ageratio);
	    ageratio = hburnlifetime/ageschw;
	    fprintf(fp,"             H-burning_lifetime/Schwarzschild_Age = %f \n", ageratio);
	    ageratio = hburnlifetime/age;
	    fprintf(fp,"       H-burning_lifetime/Greens_Main_Seq.Lifeime = %f \n", ageratio);
	    ageratio = hburnlifetime/hkage;
	    fprintf(fp,"    H-burning_lifetime/Helmholtz_contraction_time = %f \n", ageratio);
	 } // Enable white dwarf and envel.ejection report end brace.
}
// -----------------------------------------------------------------
//
// dwarfradius(dwarfmass) - Compute the radius of a white dwarf from
//                          its mass according to a table from Chandrasekhar.
//                          This is for mu_E = 2/(1 + X) = 2, when the core has
//                          been exhausted of hydrogen.
//
//         in: dwarfmass in solar masses.
//        out: dwarfradius in solar radii.
//
//    M/M_sun     R/R_sun   log(R_sun/R)
//     0.22       0.019952   1.7
//     0.40       0.015488   1.81
//     0.50	  0.013803   1.86         S. Chandrasekhar,Stellar Structure,
//     0.61	  0.012303   1.91         p.427, University of Chicago Press,
//     0.74	  0.010965   1.96         1938. Quoted from Martin Schwarzschild,
//     0.88       0.009333   2.03         Structure and Evolution of the Stars,
//     1.08	  0.007946   2.15         p.232, Dover Publications, 1965.
//     1.22       0.005495   2.26
//     1.33       0.003890   2.41
//     1.38       0.002951   2.53
//     1.44       0.00000    infinity
//
// ------------------------------------------------------------------
double dwarfradius(double dwarfmass)
{
double drad;

     if(dwarfmass < 0.22){   // analytically continue back from first table entry
	drad = 0.019952;
        return(drad);
     }
     if(dwarfmass >= 0.22 && dwarfmass < 0.4 ){
	drad = 0.019952 + (( 0.015488 - 0.019952)/(0.40 - 0.22))*(dwarfmass - 0.22);
        return(drad);
     }
     if(dwarfmass >= 0.40 && dwarfmass < 0.5){
	drad = 0.015488 + ((0.013803 - 0.015488)/(0.50 - 0.40))*(dwarfmass - 0.40);
        return(drad);
     }
     if(dwarfmass >= 0.50 && dwarfmass < 0.61){
	drad = 0.013803 + ((0.012303 - 0.013803)/(0.61 - 0.50))*(dwarfmass - 0.50);
        return(drad);
     }
     if(dwarfmass >= 0.61 && dwarfmass < 0.74){
	drad = 0.012303 + ((0.010965 - 0.012303)/(0.74 - 0.61))*(dwarfmass - 0.61);
        return(drad);
     }
     if(dwarfmass >= 0.74 && dwarfmass < 0.88){
	drad = 0.010965 + ((0.009333 - 0.010965)/(0.88 - 0.74))*(dwarfmass - 0.74);
        return(drad);
     }
     if(dwarfmass >= 0.88 && dwarfmass < 1.08){
	drad = 0.009333 + ((0.007946 - 0.009333)/(1.08 - 0.88))*(dwarfmass - 0.88);
        return(drad);
     }
     if(dwarfmass >= 1.08 && dwarfmass < 1.22){
	drad = 0.007946 + ((0.005495 - 0.007946)/(1.22 - 1.08))*(dwarfmass - 1.08);
        return(drad);
     }
     if(dwarfmass >= 1.22 && dwarfmass < 1.33){
	drad = 0.005495 + ((0.003890 - 0.005495)/(1.33 - 1.22))*(dwarfmass - 1.22);
        return(drad);
     }
     if(dwarfmass >= 1.33 && dwarfmass < 1.38){
	drad = 0.003890 + ((0.002951 - 0.003890)/(1.38 - 1.33))*(dwarfmass - 1.33);
        return(drad);
     }
     if(dwarfmass >= 1.38 && dwarfmass < 1.44){
	drad = 0.002951 + ((0.0000000001 - 0.002951)/(1.44 - 1.38))*(dwarfmass - 1.38);
        return(drad);
     }
     if(dwarfmass >= 1.44){   // analytically continue forwards from last table entry
	drad = 0.0000000001; // avoid computational crisis.
	return(drad); 
     }
}
// -----------------------------------------------------------------
//
//          dwarfm(dwarfr) - Compute the mass of a white dwarf from
//                          its radius according to a table from Chandrasekhar.
//                          This is for mu_E = 2/(1 + X) = 2, when the core has
//                          been exhausted of hydrogen.
//                          Chandrasekhars table should be revised for
//                          electrostatic screening, which lowers the
//                          pressure and the Chandrasekhar limit to 1.22
//                          solar masses, very approximately. See CHANDRA.
//
//         in: dwarfr in solar radii.
//        out: dwarfm in solar masses.
//
//    M/M_sun     R/R_sun   log(R_sun/R)
//     0.22       0.019952   1.7
//     0.40       0.015488   1.81
//     0.50	  0.013803   1.86         S. Chandrasekhar,Stellar Structure,
//     0.61	  0.012303   1.91         p.427, University of Chicago Press,
//     0.74	  0.010965   1.96         1938. Quoted from Martin Schwarzschild,
//     0.88       0.009333   2.03         Structure and Evolution of the Stars,
//     1.08	  0.007946   2.15         p.232, Dover Publications, 1965.
//     1.22       0.005495   2.26
//     1.33       0.003890   2.41
//     1.38       0.002951   2.53
//     1.44       0.00000    infinity
//
// ------------------------------------------------------------------
double dwarfm(double dwarfr)
{
double dm;

     if(dwarfr > 0.019952){   // analytically continue back from first table entry
	dm = 0.22;
        return(dm);
     }
     if(dwarfr <= 0.019952 && dwarfr > 0.015488 ){
	dm = 0.40 + (( 0.22 - 0.40)/(0.019952 - 0.015488))*(dwarfr - 0.015488);
        return(dm);
     }
     if(dwarfr <= 0.015488 && dwarfr > 0.013803){
	dm = 0.50 + ((0.40 - 0.50)/(0.015488 - 0.013803))*(dwarfr - 0.013803);
        return(dm);
     }
     if(dwarfr <= 0.013803 && dwarfr > 0.012303){
	dm = 0.61 + ((0.50 - 0.61)/(0.013803 - 0.012303))*(dwarfr - 0.012303);
        return(dm);
     }
     if(dwarfr <= 0.012303 && dwarfr > 0.010965){
	dm = 0.74 + ((0.61 - 0.74)/(0.012303 - 0.010965))*(dwarfr - 0.010965);
        return(dm);
     }
     if(dwarfr <= 0.010965 && dwarfr > 0.009333){
	dm = 0.88 + ((0.74 - 0.88)/(0.010965 - 0.009333))*(dwarfr - 0.009333);
        return(dm);
     }
     if(dwarfr <= 0.009333 && dwarfr > 0.007946){
	dm = 1.08 + ((0.88 - 1.08)/(0.009333 - 0.007946))*(dwarfr - 0.007946);
        return(dm);
     }
     if(dwarfr <= 0.007946 && dwarfr > 0.005495){
	dm = 1.22 + ((1.08 - 1.22)/(0.007946 - 0.005495))*(dwarfr - 0.005495);
        return(dm);
     }
     if(dwarfr <= 0.005495 && dwarfr > 0.003890){
	dm = 1.33 + ((1.22 - 1.33)/(0.005495 - 0.003890))*(dwarfr - 0.003890);
        return(dm);
     }
     if(dwarfr <= 0.003890 && dwarfr > 0.002951){
	dm = 1.38 + ((1.33 - 1.38)/(0.003890 - 0.002951))*(dwarfr - 0.002951);
        return(dm);
     }
     if(dwarfr <= 0.002951 && dwarfr > 0.0){
	dm = 1.44 + ((1.38 - 1.44)/(0.002951 - 0.00))*(dwarfr - 0.00);
        return(dm);
     }
     if(dwarfr <= 0.0){   // analytically continue forwards from last table entry
	dm = 1.44; // avoid computational crisis.
	return(dm); 
     }
}
//----------------------------------------------------------------
//
//  dwmass() -  White Dwarf mass = f(radius) from experimental curve-fitting
//              to measured masses and radii after Bowers & Deeming [1], pg.260.
//      in: xrstar[0] is stellar radius in meters, rsun is solar radius in meters.
//      out: returns dwarf mass in solar units.
//
//-----------------------------------------------------------------
double dwmass(void)
{
double lgmratio, dwfmratio;

     lgmratio = (-1.9846*log10(xrstar[0]/rsun)) - 3.9692; // From exp. plots,
     dwfmratio = exp(2.302585093*lgmratio);              // pg.260.
     return(dwfmratio);
}
//----------------------------------------------------------------
//
//  dwradius() -  White Dwarf radius = f(mass) from experimental curve-fitting
//              to measured masses and radii after Bowers & Deeming [1], pg.260.
//      in: dwarfmass is in solar units.
//      out: returns dwarf radius in solar units.
//
//-----------------------------------------------------------------
double dwradius(double dwarfmass)
{
double lgrratio, dwfrratio;

     lgrratio = (-0.50388*log10(dwarfmass)) - 2.00; // From exp. plots,
     dwfrratio = exp(2.302585093*lgrratio);              // pg.260.
     return(dwfrratio);
}
void depthscanner(void)   // stub for depthscanner kept in starz4sc.cpp.
{
			  // study shows the compiler requires this package seperate.
}